# coding=utf-8

import os,glob
import numpy
import fitsio
from matplotlib import pyplot as plt
import argparse

hyperparametrization = False

parser = argparse.ArgumentParser()
parser.add_argument("--hyperparametrization", help="hyperparametrization ON",action="store_true")
args = parser.parse_args()
if args.hyperparametrization:
    print("hyperparametrization turned on")
    hyperparametrization = True
else : print ("hyperparametrization turned off")

#######################################################
# LOAD DATA
#######################################################





print("-----------------------------")
print("------------LOAD DATA--------")
print("-----------------------------")

# Initialisation d'une liste pour contenir les images
# et d'une liste pour contenir les types morphologiques
raw_data =[]
type_morph =[]

 

# Définir les chemins vers les images et le catalogue,
mydataDir ='../data5/data/'
mycatalog_path =os.path.join(mydataDir,'data-COSMOS-10000-id.txt')
mypath_template ='../data5/data/image/*'
 

# Chargement des données qui nous intéressent dans le catalogue
ids, mod = numpy.loadtxt(mycatalog_path, unpack=True, usecols=(0,3))

# Chargement des images
for one in glob.glob(mypath_template):
   # Extraction de l'id à partir du nom de fichier
   # print(one)
   idi =int(one.split('/')[-1].split('_')[0])
   # print(idi)
   modi = mod[ids==idi][0]
   # print(modi)
   # On va ignorer les out layer ie les mod == 0
   if modi>0:
      # Ajout de l'image
      data = fitsio.read(one)
      raw_data.append(data)
      # Ajout du type morphologique
      type_morph.append(modi)

# Reformatage en numpy pour plus de facilité
raw_data = numpy.asarray(raw_data)
type_morph = numpy.asarray(type_morph)

#######################################################
# display
#######################################################

# Un petit graphique pour illustrer
fig = plt.figure(figsize=(7,7))
for i in range(9):
   plt.subplot(330+i+1)
   plt.imshow(raw_data[i])
   plt.title('type %d'%(type_morph[i]))
   plt.axis('off')
# Visualiser le plot
plt.show()


#######################################################
# DATA TRANSFORMATION
#######################################################

# Normalisation des images
data_scaled = numpy.asarray([(img-img.mean())/img.std()for img in raw_data])

# Transformation en 1d array
data_1d = data_scaled.reshape((data_scaled.shape[0],-1))

# Vérifions avec la dimension des données sous forme de vecteurs
print("Dimension (vecteur) :",data_1d.shape)

# Et une valeur moyenne d'environ 0
print("Valeur moyenne :",data_1d.mean())

#######################################################
# SVM
#######################################################

print("----------------------------")
print("------------SVM-------------")
print("----------------------------")

from sklearn import svm
# Création d'un classifieur SVM (Support Vector Classifier)
svm_clf = svm.SVC()

from sklearn.model_selection import train_test_split
# Séparer les jeux d'entraînement et de test
data_train, data_test, type_train, type_test = train_test_split(data_1d, type_morph, test_size=0.33)

# Apprentissage
svm_clf.fit(data_train, type_train)

# Evaluation
from sklearn import metrics
type_mod = svm_clf.predict(data_test)
print("Matrice de confusion\n", metrics.confusion_matrix(type_test, type_mod))
print("Précision, sensibilité, f-score\n", metrics.classification_report(type_test, type_mod))


if hyperparametrization == True :
	# hyper-paramètres
	from sklearn.model_selection import GridSearchCV

	# Les hyper-paramètres à explorer :
	# * les quatre noyaux implémentés dans SVC
	# * 4 valeurs de marge
	parameters ={'kernel':('linear','poly','rbf','sigmoid'),'C':[1,5,10,20]}

	# !!!! Attention adapter njob à votre machine !!!!!
	grid_clf = GridSearchCV(svm_clf, parameters, n_jobs=8)
	grid_clf.fit(data_train, type_train)
	type_mod = grid_clf.best_estimator_.predict(data_test)
	print("Meilleur classifieur\n", grid_clf.best_estimator_)
	print("Matrice de confusion\n", metrics.confusion_matrix(type_test, type_mod))
	print("Précision, sensibilité, f-score\n", metrics.classification_report(type_test, type_mod))



#######################################################
# RandomForestClassifier
####################################################### 

print("------------------------------------------")
print("------------RandomForestClassifier--------")
print("------------------------------------------")

from sklearn.ensemble import RandomForestClassifier

 
# Une bonne valeur pour n_estimator est en général sqrt(m)
# m étant la taille des données
# ici n_estimators=100 pour nos images de 100 x 100
rf_clf = RandomForestClassifier(n_estimators=100)
rf_clf.fit(data_train, type_train)
type_mod = rf_clf.predict(data_test)
print("Matrice de confusion\n", metrics.confusion_matrix(type_test, type_mod))
print("Précision, sensibilité, f-score\n", metrics.classification_report(type_test, type_mod))

#######################################################
# AdaBoostClassifier
####################################################### 


print("--------------------------------------")
print("------------AdaBoostClassifier--------")
print("--------------------------------------")
from sklearn.ensemble import AdaBoostClassifier

ab_clf = AdaBoostClassifier(n_estimators=100)
ab_clf.fit(data_train, type_train)
type_mod = ab_clf.predict(data_test)
print("Matrice de confusion\n", metrics.confusion_matrix(type_test, type_mod))
print("Précision, sensibilité, f-score\n", metrics.classification_report(type_test, type_mod))


#######################################################
# GradientBoostingClassifier
####################################################### 

print("----------------------------------------------")
print("------------GradientBoostingClassifier--------")
print("----------------------------------------------")

from sklearn.ensemble import GradientBoostingClassifier

gb_clf = GradientBoostingClassifier(n_estimators=100)
gb_clf.fit(data_train, type_train)
type_mod = gb_clf.predict(data_test)

print("Matrice de confusion\n", metrics.confusion_matrix(type_test, type_mod))
print("Précision, sensibilité, f-score\n", metrics.classification_report(type_test, type_mod))

#######################################################
# MLPClassifier
####################################################### 


print("---------------------------------")
print("------------MLPClassifier--------")
print("---------------------------------")

from sklearn.neural_network import MLPClassifier

# 100 neurones cachés
mlp_clf = MLPClassifier(hidden_layer_sizes=100)
mlp_clf.fit(data_train, type_train)
type_mod = mlp_clf.predict(data_test)
print("Matrice de confusion\n", metrics.confusion_matrix(type_test, type_mod))
print("Précision, sensibilité, f-score\n", metrics.classification_report(type_test, type_mod))


if hyperparametrization == True :
	# hyper-paramètres
	# Boucle sur quelques valeurs de nombre de neurones par couche
	for n_neu in[50,100,200]:
		# Boucle sur le nombre de couche
		for n_lay in[1,2,3]:
			# Pour définir plusieurs couches, il suffit de passer au MLPClassifier
			# un hidden_layer_sizes sous forme de tuple
			layer = tuple([n_neu for n in range(n_lay)])
			print("Layers: %s"%str(layer), end=' ')

			mlp = MLPClassifier(hidden_layer_sizes=layer)
			mlp.fit(data_train, type_train)
			print("Test set score: %f"% mlp.score(data_test, type_test))



#######################################################
# PREDICTION
####################################################### 


print("---------------------------------")
print("------------PREDICTION-----------")
print("---------------------------------")

new_data =[]

 

# À remplacer par vos chemins bien sûr!

mynewimgDir ='../data5/data/new_image/'
mypath_template =os.path.join(mynewimgDir,'*_acs_I_mosaic_30mas_sci.fits')

 

# Chargement des images
for one in glob.glob(mypath_template):
	# Chargement de l'image
	img = fitsio.read(one)
	# On ajoute l'image à la liste
	# et on en profite pour la normaliser
	new_data.append((img-img.mean())/img.std())

# Reformatage en numpy pour plus de facilité
new_data = numpy.asarray(new_data)
new_1d = new_data.reshape((new_data.shape[0],-1))

# On test ici le RandomForestClassifier (rf)
type_rf = rf_clf.predict(new_1d)

# Réseau de neurones (MLP)
type_mlp = mlp_clf.predict(new_1d)


# Graphique pour illustrer pour comparer les 2 classifiers
fig = plt.figure(figsize=(7,7))
for i in range(9):
	plt.subplot(330+i+1)
	plt.axis('off')
	plt.imshow(new_data[i])
	if type_rf[i] == type_mlp[i]:
 		plt.title('type_rf %d ; type_mlp %d ' % (type_rf[i], type_mlp[i]))
	else :	
 		plt.title('type_rf %d ; type_mlp %d ' % (type_rf[i], type_mlp[i]), color='C1')

plt.show()




